###############################################################################
#	Filename:	Starbase12.py
#	
#	Confidential and Proprietary, Copyright 2001 by Totally Games
#	
#	Starbase 12 region. Also contains Starbase 12 control room character set.
#	
#	Created:	1/5/01 -	Alberto Fonseca
###############################################################################

import App
import Bridge.BridgeUtils
import MissionLib
import Bridge.Characters.Graff
import Bridge.HelmMenuHandlers
import Tactical.LensFlares

#NonSerializedObjects = ( "debug", )

#debug = App.CPyDebug(__name__).Print
#debug("Loading " + __name__ + " module...")

g_idDockAI = None

###############################################################################
#	Initialize()
#	
#	Initialize the Starbase 12 set.
#	
#	Args:	pSet	- The Starbase 12 set.
#	
#	Return:	none
###############################################################################
def Initialize(pSet):
	SetupEventHandlers(pSet)

	# Add a sun, far far away
	pSun = App.Sun_Create(1000.0, 1000, 500, "data/Textures/SunBlueWhite.tga", "data/Textures/Effects/SunFlaresWhite.tga")
	pSet.AddObjectToSet(pSun, "Sun")
	
	# Place the object at the specified location.
	pSun.PlaceObjectByName( "Sun" )
	pSun.UpdateNodeOnly()

	# Builds a Blue lens flare, attached to the sun
	Tactical.LensFlares.BlueLensFlare(pSet, pSun)

	######
	# Create the Planet
	pPlanet = App.Planet_Create(400.0, "data/models/environment/planet.nif")
	pSet.AddObjectToSet(pPlanet, "New Holland")

	# Place the object at the specified location.
	pPlanet.PlaceObjectByName( "Planet1" )
	pPlanet.UpdateNodeOnly()

	# X_TheUnknown_X: Let's give New Holland an atmosphere. It's habitable, so the people will want something to breathe...
	# If the script doesn't work (hopefully due to lack of NanoFX), print an error message.
	try:
                import Custom.NanoFXv2.NanoFX_Lib
                Custom.NanoFXv2.NanoFX_Lib.CreateAtmosphereFX(pPlanet, "data/models/environment/planet.nif", "Class-M")
        except:
                print "Starbase 12 System: NanoFX atmospheres not insalled or not functioning."

        ######
        # X_TheUnknown_X: My own personal addition is to add a moon, it fills up the system.
        # Let's call it Rhine. The river Rhine runs though Holland.
	# Create the Planet
	pMoon = App.Planet_Create(65, "data/models/environment/Charon.nif")
	pSet.AddObjectToSet(pMoon, "Rhine")

	# Place the object at the specified location.
	pMoon.PlaceObjectByName( "Moon1" )
	pMoon.UpdateNodeOnly()

	######
	# X_TheUnknown_X: The below code should remove the stock Starbase 12
	import loadspacehelper, MissionLib
	pSet = App.g_kSetManager.GetSet("Starbase12")
	pSet.RemoveObjectFromSet("Starbase 12")
	# Next, load my starbase - and then all the other ships to give the system some life.
	# The multiplayer checking is probably not required. In fact, it might remove SB12 from MP altogether.
	# Not a concern, really beacuse MP won't work with this mod unless all players have it.
	if (App.g_kUtopiaModule.IsHost ()) or not (App.g_kUtopiaModule.IsMultiplayer()):
                # Create ships
                pSB12  = loadspacehelper.CreateShip("TUFedStardock", pSet, "Starbase 12", "Starbase12 Location")
                pAkira =  loadspacehelper.CreateShip("Akira", pSet, "USS Devore", "Akira Location")
                pNeb =  loadspacehelper.CreateShip("Nebula", pSet, "USS Khitomer", "Neb Location")
                pGalaxy =  loadspacehelper.CreateShip("Galaxy", pSet, "USS San Fransisco", "Galaxy Location")
                pTransport =  loadspacehelper.CreateShip("Transport", pSet, "CVN-65", "Transport Location")
                pShuttle1 =  loadspacehelper.CreateShip("DanubemkI", pSet, "SB12 03", "Shuttle1 Location")
                pShuttle2 =  loadspacehelper.CreateShip("Shuttle", pSet, "SB12 07", "Shuttle2 Location")

        if (App.g_kUtopiaModule.IsMultiplayer()):
                pMission = MissionLib.GetMission()
                SetupEventHandlers(pMission)
        else:
                # Setup Affiliations
                global pFriendlies
		global pEnemies
		pGame = App.Game_GetCurrentGame()
		pEpisode = pGame.GetCurrentEpisode()
		pMission = pEpisode.GetCurrentMission()
		pFriendlies = MissionLib.GetFriendlyGroup()
		pFriendlies.AddName("Starbase 12")
		pFriendlies.AddName("USS Devore")
		pFriendlies.AddName("USS Khitomer")
		pFriendlies.AddName("USS San Fransisco")
		pFriendlies.AddName("CVN-65")
		pFriendlies.AddName("SB12 03")
		pFriendlies.AddName("SB12 07")

		# Set up AIs
		import StarbaseFriendlyAI, DefendAI, RouteAI, RouteWaypointAI, OrbitAI
		pSB12.SetAI(StarbaseFriendlyAI.CreateAI(pSB12))
		pAkira.SetAI(DefendAI.CreateAI(pAkira, pSB12))
		pNeb.SetAI(DefendAI.CreateAI(pNeb, pAkira))
		pGalaxy.SetAI(DefendAI.CreateAI(pGalaxy, pSB12))
		pTransport.SetAI(RouteWaypointAI.CreateAI(pTransport, "Transport Location", "TransportRouteEnd"))
		pShuttle1.SetAI(RouteAI.CreateAI(pShuttle1, pMoon, pPlanet))
		pShuttle2.SetAI(OrbitAI.CreateAI(pShuttle2, pPlanet))

###############################################################################
#	SetupEventHandlers()
#	
#	Set up event handlers used by the Starbase 12 set.
#	
#	Args:	pSet	- The Starbase 12 set.
#	
#	Return:	none
###############################################################################
def SetupEventHandlers(pSet):
	pGame = App.Game_GetCurrentGame()
	pEpisode = pGame.GetCurrentEpisode()

	# Ship entrance event
	App.g_kEventManager.AddBroadcastPythonFuncHandler(App.ET_ENTERED_SET, pSet,
														__name__ + ".EnterSet")
	# Ship exit event
	App.g_kEventManager.AddBroadcastPythonFuncHandler(App.ET_EXITED_SET, pSet,
														__name__ + ".ExitSet")
	
###############################################################################
#	EnterSet(pObject, pEvent)
#	
#	Event handler for player's ship entering this set.
#	Create the Starbase 12 Control Room Set when the player enters.
#	
#	Args:	
#	
#	Return:	
###############################################################################
def EnterSet(pObject, pEvent):
	try:
		# Get ship entering set.
		pShip = App.ShipClass_Cast(pEvent.GetDestination())

		# Get player.
		pPlayer = App.Game_GetCurrentPlayer()

		if pShip.GetObjID() != pPlayer.GetObjID():
			# This ship isn't the player's ship.  We're done.
			return
	except AttributeError:
		# Ship or Player is None.  Nothing to do here.
		return

	# The player is entering a set.  If they're entering the Starbase 12
	# set, setup the SB12 Control Room.
	if pShip.GetContainingSet() and pShip.GetContainingSet().GetName() == "Starbase12":
		SetupGraffSet()

###############################################################################
#	SetupGraffSet()
#	
#	Setup the Starbase 12 Control Room set, so Graff can say something
#	if the player chooses to dock.  If the set already exists, this
#	does nothing.
#	
#	Args:	None
#	
#	Return:	None
###############################################################################
def SetupGraffSet():
	pSB12Set = App.g_kSetManager.GetSet("FedOutpostSet_Graff")
	if not pSB12Set:
		pSB12Set = MissionLib.SetupBridgeSet("FedOutpostSet_Graff", "data/Models/Sets/FedOutpost/fedoutpost.nif", -30, 65, -1.55)
	if not App.CharacterClass_GetObject(pSB12Set, "Graff"):
		MissionLib.SetupCharacter("Bridge.Characters.Graff", "FedOutpostSet_Graff")

	# Enable dock button.
	pButton = Bridge.BridgeUtils.GetDockButton()
	if(pButton):
		pButton.SetEnabled()


###############################################################################
#	ExitSet(pObject, pEvent)
#	
#	Event handler for player's ship exiting this set.
#	Delete the Starbase 12 Control Room Set when the player enters.
#
#	Args:	
#	
#	Return:	
###############################################################################
def ExitSet(pObject, pEvent):
	try:
		# Get ship exiting set.
		pShip = App.ShipClass_Cast(pEvent.GetDestination())

		# Get player.
		pPlayer = App.Game_GetCurrentPlayer()

		if pShip.GetObjID() != pPlayer.GetObjID():
			# This ship isn't the player's ship.  We're done.
			return
	except AttributeError:
		# Ship or Player is None.  Nothing to do here.
		return

	# The player's ship is exiting a set.  Check if it's exiting
	# the Starbase 12 set...
	if pEvent.GetCString() == "Starbase12":
		# Delete the Starbase 12 Control Room Set.
		pStarbaseControlSet = App.g_kSetManager.GetSet("FedOutpostSet_Graff")
		if pStarbaseControlSet:
			App.g_kSetManager.DeleteSet("FedOutpostSet_Graff")
	
		Bridge.Characters.Graff.RemoveEventHandlers()

		# Disable dock button.
		pButton = Bridge.BridgeUtils.GetDockButton()
		if(pButton):
			pButton.SetDisabled()

		
###############################################################################
#	DockStarbase():
#	
#	Make Player's ship dock/undock with Starbase 12.
#	
#	Args:	none
#	
#	Return:	none
###############################################################################
def DockStarbase():
	# Get Player.
	pPlayer = MissionLib.GetPlayer()
	if(pPlayer is None):
		return
	
	# Get Starbase 12 Set.	
	pStarbase12Set = App.g_kSetManager.GetSet("Starbase12")
	if(pStarbase12Set is None):
#		debug("DockStarbase() unable to get Starbase 12 set.")
		return

	# Get Starbase 12.
	pStarbase12 = pStarbase12Set.GetObject("Starbase 12")
	if(pStarbase12 is None):
#		debug("DockStarbase() unable to get Starbase 12.")
		return

	# Check if there's a special action for Graff when the
	# player docks with the starbase.
	global g_idGraffAction
	try:
		pGraffAction = App.TGAction_Cast( App.TGObject_GetTGObjectPtr( g_idGraffAction ) )
	except NameError:
		pGraffAction = None

	try:
		bFadeEnd = g_bFadeGraffEnd
	except NameError:
		bFadeEnd = 1

	# Set AI for docking/undocking.
	import AI.Compound.DockWithStarbase
	MissionLib.SetPlayerAI("Helm", AI.Compound.DockWithStarbase.CreateAI(pPlayer, pStarbase12, pGraffAction, NoRepair = not g_bRepairsEnabled, FadeEnd = bFadeEnd))

###############################################################################
#	StarbaseRepairsEnabled
#	
#	When a ship docks with the starbase, this sets whether or not
#	that ship is repaired.
#	
#	Args:	bRepairsEnabled	- 0 for no repairs, 1 to allow repairs.
#	
#	Return:	None
###############################################################################
g_bRepairsEnabled = 1
def StarbaseRepairsEnabled(bRepairsEnabled):
	global g_bRepairsEnabled
	g_bRepairsEnabled = bRepairsEnabled

###############################################################################
#	SetGraffDockingAction
#	
#	Set a special TGAction to replace Commander Graff's normal
#	response when the player docks with Starbase 12.
#	
#	Args:	pAction	- The replacement action.  If this is None,
#					  Graff's normal behavior is restored.
#			bFadeEnd- True if the camera should fade to black after
#					  Graff's cutscene finishes, as the camera switches
#					  to the Sovereign flying out of the starbase.
#	
#	Return:	None
###############################################################################
def SetGraffDockingAction(pAction = None, bFadeEnd = 1):
	global g_idGraffAction
	global g_bFadeGraffEnd
	g_bFadeGraffEnd = bFadeEnd
	# If there was a replacement action before, make sure it's cleaned up.
	try:
		pGraffAction = App.TGAction_Cast( App.TGObject_GetTGObjectPtr( g_idGraffAction ) )
		if pGraffAction:
			pGraffAction.Completed()
	except NameError: pass

	if pAction:
		g_idGraffAction = pAction.GetObjID()
	else:
		g_idGraffAction = App.NULL_ID
